package fundamentos;

/* As seguintes linhas permitem o uso das classes de entrada e sada e de processamento
   de tokens */
import java.io.*;
import java.util.*;

/**
 * A classe <tt>Keyboard</tt> permite a leitura de dados de tipos nativos e de
 * instncias da classe <tt>String</tt> a partir do teclado. Basicamente esta classe 
 * encapsula o funcionamento do Stream <tt>System.in</tt>, que pode ser usado como 
 * entrada padro de dados.
 * <p>                         
 * Esta classe  baseada na classe <tt>EasyIn</tt>, de Peter van der Linden. As
 * principais modificaes foram:
 * <ul>
 * <li>Os mtodos so estticos para facilitar o uso da classe
 * <li>Criao de mtodos sobrecarregados para que valores <i>default</i> possam ser 
 *     usados
 * <li>Melhor tratamento das excees
 * <li>Reconhecimento de vrios valores como booleanos e capacidade de leitura de 
 *     <tt>NaNs</tt> e infinitos para valores dos tipos <tt>float</tt> e 
 *     <tt>double</tt>
 * <li>Descrio dos detalhes de implementao com comentrios
 * </ul>                        
 * A classe permite o uso de arquivos de respostas (<i>"answer files"</i>), que so 
 * arquivos contendo os dados que deveriam ser entrados pelo teclado. A entrada destes 
 * dados pode ser simulada atravs de redirecionamento de arquivos. Se, por exemplo, 
 * tivermos uma aplicao (classe) <tt>DemoDataKeyboard</tt> e quisermos usar um arquivo 
 * de respostas chamado <tt>dados.dat</tt> que j contm os dados que devem ser 
 * digitados, podemos executar o comando <tt>java DemoDataKeyboard &lt; dados.dat </tt>
 * para ler os dados do arquivo em vez do teclado. O comando ser o mesmo para 
 * os sistemas operacionais Windows e Linux.
 * <p>
 * O suporte a arquivos de respostas  limitado, somente funcionar se todos os dados
 * a serem digitados forem entrados no arquivo de respostas, um em cada linha. 
 * Excees ocorrero caso estas regras no sejam seguidas.
 * <p>
 * A mensagem original de copyright de Peter van de Linden  mostrada ao final desta
 * listagem. Vale a pena notar que a sintaxe proposta e mostrada na mensagem  
 * diferente da usada pela classe <tt>Keyboard</tt>.
 * <p>                   
 * O livro <b>"An Introduction to Computer Science Using Java"</b>, de Samuel N. Kamin,
 * M. Dennis Mickunas e Edward M. Reingold (editora McGraw-Hill, ISBN 0-07-034224-5)
 * tambm apresenta uma classe <tt>Keyboard</tt>, cujo funcionamento e cdigo so 
 * diferentes desta.
 *   
 * @author Rafael Santos
 * @version 1.2
 */
public class Keyboard
  {
 /**
  * Declarao dos campos privados desta classe.
  */
  // O campo is  uma instncia da classe InputStreamReader que ser construda a 
  // partir do stream System.in. Note que j que esta classe s tem mtodos estticos,
  // no ter construtor, mas podemos inicializar instncias de classes da forma mostrada 
  // abaixo.
  private static InputStreamReader is = new InputStreamReader( System.in );
  // O campo br  uma instncia da classe BufferedReader que usa 
  // is como argumento para seu construtor.
  private static BufferedReader br = new BufferedReader( is );
  // O campo st  uma instncia da classe StringTokenizer que ser 
  // usada em vrios mtodos nesta classe.
  private static StringTokenizer st;
  // O campo nt  uma instncia da classe String que ser usada em 
  // vrios mtodos nesta classe.
  private static String nt;
  // O campo debug estabelece se mensagens de erro de converso e leitura 
  // devem ser mostradas ou no. O campo no pode ser modificado diretamente, 
  // somente atravs de mtodos especficos descritos abaixo.
  private static boolean debug = false;

 /**
  * O mtodo <tt>getToken</tt> l uma string do <tt>BufferedReader</tt> associado com 
  * a entrada padro e retorna uma instncia de <tt>StringTokenizer</tt> contendo os 
  * <i>tokens</i> criados a partir da linha lida. O mtodo  declarado como 
  * <tt>private</tt> pois s dever ser chamado a partir de outros mtodos da classe.
  * @return uma instncia de <tt>StringTokenizer</tt> contendo os <i>tokens</i> 
  *         obtidos da linha lida
  * @exception IOException se um erro de entrada e sada ocorrer.
  * @exception NullPointerException se uma string vazia for lida e tentarmos construir
  *                                 uma instncia de <tt>StringTokenizer</tt> com ela
  * @see java.util.StringTokenizer <tt>StringTokenizer</tt>
  */
  private static StringTokenizer getToken() throws IOException, NullPointerException 
    {
    String s = br.readLine();
    return new StringTokenizer(s);
    } // fim do mtodo getToken

 /**
  * O mtodo <tt>readBoolean</tt> l e retorna um valor do tipo <tt>boolean</tt>. 
  * Este mtodo simplesmente chama o mtodo <tt>readBoolean</tt> com argumentos, 
  * descrito abaixo, considerando o valor <i>default</i> como sendo <tt>true</tt>.
  * @return o valor do tipo boolean lido (ou, em caso de erro de leitura, 
  *         <tt>true</tt>)
  */
  public static boolean readBoolean() 
    {
    return readBoolean(true);
    }

 /**
  * O mtodo <tt>readBoolean</tt> l e retorna um valor do tipo <tt>boolean</tt>. Este 
  * mtodo tenta pegar o prximo <i>token</i> a ser lido da entrada padro e o compara
  *  com algumas strings constantes para verificar a igualdade, retornando <tt>true</tt> 
  * ou <tt>false</tt> dependendo do resultado da comparao.
  * @param defaultvalue o valor <i>default</i> caso no seja possvel ler e decodificar
  *                     um valor booleano
  * @return o valor do tipo <tt>boolean</tt> lido (ou, em caso de erro de leitura, igual ao 
  *         argumento passado para o mtodo)
  */
  public static boolean readBoolean(boolean defaultvalue) 
    {
    try 
      {
      st = getToken(); // pega os tokens a partir da linha lida
      nt = st.nextToken(); // e a primeira string do token.
      // Retorna true se a string for igual a...
      if (nt.equalsIgnoreCase("true") ||        // true
	  nt.equalsIgnoreCase("t") ||           // t
	  nt.equalsIgnoreCase("yes") ||         // yes
	  nt.equalsIgnoreCase("y") ||           // y
	  nt.equalsIgnoreCase("on") ||          // on
	  nt.equalsIgnoreCase("v") ||           // v
	  nt.equalsIgnoreCase("s") ||           // s
	  nt.equalsIgnoreCase("sim") ||         // sim
	  nt.equalsIgnoreCase("verdadeiro"))    // verdadeiro
	  return true; 
      // Retorna false se a string for igual a...
      else if (nt.equalsIgnoreCase("false") ||  // false
	       nt.equalsIgnoreCase("f") ||      // f
	       nt.equalsIgnoreCase("no") ||     // no
	       nt.equalsIgnoreCase("n") ||      // n
	       nt.equalsIgnoreCase("off") ||    // off
	       nt.equalsIgnoreCase("nao") ||    // nao
	       nt.equalsIgnoreCase("no") ||    // no
	       nt.equalsIgnoreCase("falso"))    // falso
	  return false; 
      // Se no for nenhum dos valores reconhecidos, retorna o default
      else return defaultvalue;
      }
    catch (IOException ioe) // se houver algum erro de leitura
      {
      if (debug) // se for pedida a impresso de mensagens de erro
	System.err.println("KEYBOARD:: Erro de entrada e sada lendo um boolean. "+
			   "Retorna "+defaultvalue);
      return defaultvalue; // retorna o valor default
      }
    catch (NoSuchElementException nsee) // se no houver tokens
      {
      if (debug) // se for pedida a impresso de mensagens de erro
	System.err.println("KEYBOARD:: Entrada no contm um boolean. "+
			   "Retorna "+defaultvalue);
      return defaultvalue; // retorna o valor default
      }
    } // fim do mtodo readBoolean

 /**
  * O mtodo <tt>readByte</tt> l e retorna um valor do tipo <tt>byte</tt>. Este 
  * mtodo simplesmente chama o mtodo <tt>readByte</tt> com argumentos, descrito 
  * abaixo, considerando o valor <i>default</i> como sendo <tt>(byte)0</tt>.
  * @return o valor do tipo <tt>byte</tt> lido (ou, em caso de erro de leitura, 
  *         <tt>(byte)0</tt>)
  */
  public static byte readByte() 
    {
    return readByte((byte)0);
    } // fim do mtodo readByte

 /**
  * O mtodo <tt>readByte</tt> l e retorna um valor do tipo <tt>byte</tt>. Este 
  * mtodo tenta pegar o prximo <i>token</i> a ser lido da entrada padro e o 
  * passa como argumento para o mtodo <tt>parseByte</tt> da classe <tt>Byte</tt>, 
  * que tenta convert-lo. Se a leitura e converso puderem ser feitas, um valor
  * do tipo <tt>byte</tt>  retornado, caso contrrio o valor <i>default</i> 
  * (passado como argumento)  retornado.
  * @param defaultvalue o valor <i>default</i> caso no seja possvel ler e 
  *                     decodificar um valor do tipo <tt>byte</tt>
  * @return o valor do tipo <tt>byte</tt> lido (ou, em caso de erro de leitura, 
  *         igual ao argumento passado para o mtodo)
  */
  public static byte readByte(byte defaultvalue) 
    {
    try 
      {
      st = getToken(); // pega os tokens a partir da linha lida
      nt = st.nextToken(); // e a primeira string do token.
      // retorna o valor processado pela classe Byte
      return Byte.parseByte(nt);
      }
    catch (IOException ioe) // se houver algum erro de leitura
      {
      if (debug) // se for pedida a impresso de mensagens de erro
	System.err.println("KEYBOARD:: Erro de entrada e sada lendo um byte. "+
			   "Retorna "+defaultvalue);
      return defaultvalue; // retorna o valor default
      }
    catch (NumberFormatException nfe) // se houver algum erro de converso
      {
      if (debug) // se for pedida a impresso de mensagens de erro
	System.err.println("KEYBOARD:: Erro de converso de "+nt+" para um byte. "+
			   "Retorna "+defaultvalue);
      return defaultvalue;
      }
    catch (NoSuchElementException nsee) // se no houver tokens
      {
      if (debug) // se for pedida a impresso de mensagens de erro
	System.err.println("KEYBOARD:: Entrada no contm um byte. "+
			   "Retorna "+defaultvalue);
      return defaultvalue; // retorna o valor default
      }
    } // fim do mtodo readByte

 /**
  * O mtodo <tt>readShort</tt> l e retorna um valor do tipo <tt>short</tt>. 
  * Este mtodo simplesmente chama o mtodo <tt>readShort</tt> com argumentos, 
  * descrito abaixo, considerando o valor <i>default</i> como sendo <tt>(short)0</tt>.
  * @return o valor do tipo <tt>short</tt> lido (ou, em caso de erro de leitura, 
  *         <tt>(short)0</tt>)
  */
  public static short readShort() 
    {
    return readShort((short)0);
    } // fim do mtodo readShort

 /**
  * O mtodo <tt>readShort</tt> l e retorna um valor do tipo <tt>short</tt>. Este 
  * mtodo tenta pegar o prximo <i>token</i> a ser lido da entrada padro e o passa 
  * como argumento para o mtodo <tt>parseShort</tt> da classe <tt>Short</tt>, que 
  * tenta convert-lo. Se a leitura e converso puderem ser feitas, um valor do tipo 
  * <tt>short</tt>  retornado, caso contrrio o valor <i>default</i> (passado 
  * como argumento)  retornado.
  * @param defaultvalue o valor <i>default</i> caso no seja possvel ler e 
  *                     decodificar um valor do tipo short
  * @return o valor do tipo <tt>short</tt> lido (ou, em caso de erro de leitura, 
  *         igual ao argumento passado para o mtodo)
  */
  public static short readShort(short defaultvalue) 
    {
    try 
      {
      st = getToken(); // pega os tokens a partir da linha lida
      nt = st.nextToken(); // e a primeira string do token.
      // retorna o valor processado pela classe Short
      return Short.parseShort(nt);
      }
    catch (IOException ioe) // se houver algum erro de leitura
      {
      if (debug) // se for pedida a impresso de mensagens de erro
	System.err.println("KEYBOARD:: Erro de entrada e sada lendo um short. "+
			   "Retorna "+defaultvalue);
      return defaultvalue; // retorna o valor default
      }
    catch (NumberFormatException nfe) // se houver algum erro de converso
      {
      if (debug) // se for pedida a impresso de mensagens de erro
	System.err.println("KEYBOARD:: Erro de converso de "+nt+" para um short. "+
			   "Retorna "+defaultvalue);
      return defaultvalue;
      }
    catch (NoSuchElementException nsee) // se no houver tokens
      {
      if (debug) // se for pedida a impresso de mensagens de erro
	System.err.println("KEYBOARD:: Entrada no contm um short. "+
			   "Retorna "+defaultvalue);
      return defaultvalue; // retorna o valor default
      }
    } // fim do mtodo readShort

 /**
  * O mtodo <tt>readInt</tt> l e retorna um valor do tipo <tt>int</tt>. Este
  * mtodo simplesmente chama o mtodo <tt>readInt</tt> com argumentos, descrito 
  * abaixo, considerando o valor <i>default</i> como sendo <tt>0</tt>.
  * @return o valor do tipo <tt>int</tt> lido (ou, em caso de erro de leitura,
  *         </tt>0</tt>)
  */
  public static int readInt() 
    {
    return readInt(0);
    } // fim do mtodo readInt

 /**
  * O mtodo <tt>readInt</tt> l e retorna um valor do tipo <tt>int</tt>. Este
  * mtodo tenta pegar o prximo <i>token</i> a ser lido da entrada padro e o 
  * passa como argumento para o mtodo <tt>parseInt<tt> da classe <tt>Integer</tt>,
  * que tenta convert-lo. Se a leitura e converso puderem ser feitas, um valor 
  * do tipo <tt>int</tt>  retornado, caso contrrio o valor <i>default</i> 
  * (passado como argumento)  retornado.
  * @param defaultvalue o valor <i>default</i> caso no seja possvel ler e 
  *                     decodificar um valor do tipo <tt>int</tt>
  * @return o valor do tipo <tt>int</tt> lido (ou, em caso de erro de leitura, 
  *         igual ao argumento passado para o mtodo)
  */
  public static int readInt(int defaultvalue) 
    {
    try 
      {
      st = getToken(); // pega os tokens a partir da linha lida
      nt = st.nextToken(); // e a primeira string do token.
      // retorna o valor processado pela classe Integer
      return Integer.parseInt(nt);
      }
    catch (IOException ioe) // se houver algum erro de leitura
      {
      if (debug) // se for pedida a impresso de mensagens de erro
	System.err.println("KEYBOARD:: Erro de entrada e sada lendo um int. "+
			   "Retorna "+defaultvalue);
      return defaultvalue; // retorna o valor default
      }
    catch (NumberFormatException nfe) // se houver algum erro de converso
      {
      if (debug) // se for pedida a impresso de mensagens de erro
	System.err.println("KEYBOARD:: Erro de converso de "+nt+" para um int. "+
			   "Retorna "+defaultvalue);
      return defaultvalue;
      }
    catch (NoSuchElementException nsee) // se no houver tokens
      {
      if (debug) // se for pedida a impresso de mensagens de erro
	System.err.println("KEYBOARD:: Entrada no contm um int. "+
			   "Retorna "+defaultvalue);
      return defaultvalue; // retorna o valor default
      }
    } // fim do mtodo readInt

 /**
  * O mtodo <tt>readLong</tt> l e retorna um valor do tipo <tt>long</tt>. Este
  * mtodo simplesmente chama o mtodo <tt>readLong</tt> com argumentos, descrito
  * abaixo, considerando o valor <i>default</i> como sendo <tt>0l</tt>.
  * @return o valor do tipo <tt>long</tt> lido (ou, em caso de erro de leitura, 
  *         <tt>0l</tt>)
  */
  public static long readLong() 
    {
    return readLong(0l);
    } // fim do mtodo readLong

 /**
  * O mtodo <tt>readLong</tt> l e retorna um valor do tipo <tt>long</tt>. Este 
  * mtodo tenta pegar o prximo <i>token</i> a ser lido da entrada padro e o 
  * passa como argumento para o mtodo <tt>parseLong</tt> da classe <tt>Long</tt>,
  * que tenta convert-lo. Se a leitura e converso puderem ser feitas, um valor 
  * do tipo <tt>long</tt>  retornado, caso contrrio o valor <i>default</i> 
  * (passado como argumento)  retornado.
  * @param defaultvalue o valor <i>default</i> caso no seja possvel ler e 
  *                     decodificar um valor do tipo <tt>long</tt>
  * @return o valor do tipo <tt>long</tt> lido (ou, em caso de erro de leitura, 
  *         igual ao argumento passado para o mtodo)
  */
  public static long readLong(long defaultvalue) 
    {
    try 
      {
      st = getToken(); // pega os tokens a partir da linha lida
      nt = st.nextToken(); // e a primeira string do token.
      // retorna o valor processado pela classe Long
      return Long.parseLong(nt);
      }
    catch (IOException ioe) // se houver algum erro de leitura
      {
      if (debug) // se for pedida a impresso de mensagens de erro
	System.err.println("KEYBOARD:: Erro de entrada e sada lendo um long. "+
			   "Retorna "+defaultvalue);
      return defaultvalue; // retorna o valor default
      }
    catch (NumberFormatException nfe) // se houver algum erro de converso
      {
      if (debug) // se for pedida a impresso de mensagens de erro
	System.err.println("KEYBOARD:: Erro de converso de "+nt+" para um long. "+
			   "Retorna "+defaultvalue);
      return defaultvalue;
      }
    catch (NoSuchElementException nsee) // se no houver tokens
      {
      if (debug) // se for pedida a impresso de mensagens de erro
	System.err.println("KEYBOARD:: Entrada no contm um long. "+
			   "Retorna "+defaultvalue);
      return defaultvalue; // retorna o valor default
      }
    } // fim do mtodo readLong

 /**
  * O mtodo <tt>readFloat</tt> l e retorna um valor do tipo <tt>float</tt>. Este 
  * mtodo simplesmente chama o mtodo <tt>readFloat</tt> com argumentos, descrito 
  * abaixo, considerando o valor <i>default</i> como sendo <tt>0f</tt>.
  * @return o valor do tipo <tt>float</tt> lido (ou, em caso de erro de leitura, 
  *         <tt>0f</tt>)
  */
  public static float readFloat() 
    {
    return readFloat(0f);
    } // fim do mtodo readFloat

 /**
  * O mtodo <tt>readFloat</tt> l e retorna um valor do tipo <tt>float</tt>. Este 
  * mtodo tenta pegar o prximo <i>token</i> a ser lido da entrada padro e o 
  * analisa, verificando se aparentemente  <tt>NaN</tt> ou infinito. Se no for, 
  * passa o valor lido como argumento para o mtodo <tt>parseFloat</tt> da classe 
  * <tt>Float</tt>, que tenta convert-lo. Se a leitura e converso puderem ser 
  * feitas, um valor do tipo <tt>float</tt>  retornado, caso contrrio o valor
  * <i>default</i> (passado como argumento)  retornado.
  * @param defaultvalue o valor <i>default</i> caso no seja possvel ler e decodificar
  *                     um valor do tipo <tt>float</tt>
  * @return o valor do tipo <tt>float</tt> lido (ou, em caso de erro de leitura, 
  *         igual ao argumento passado para o mtodo)
  */
  public static float readFloat(float defaultvalue) 
    {
    try 
      {
      st = getToken(); // pega os tokens a partir da linha lida
      nt = st.nextToken(); // e a primeira string do token.
      // Verifica se o valor  aparentemente NaN ou infinito
      if (nt.toLowerCase().startsWith("nan")) return Float.NaN;
      else if (nt.toLowerCase().startsWith("inf")) return Float.POSITIVE_INFINITY;
      else if (nt.toLowerCase().startsWith("+inf")) return Float.POSITIVE_INFINITY;
      else if (nt.toLowerCase().startsWith("-inf")) return Float.NEGATIVE_INFINITY;
      // Retorna o valor processado pela classe Float
      else return Float.parseFloat(nt);
      }
    catch (IOException ioe) // se houver algum erro de leitura
      {
      if (debug) // se for pedida a impresso de mensagens de erro
	System.err.println("KEYBOARD:: Erro de entrada e sada lendo um float. "+
			   "Retorna "+defaultvalue);
      return defaultvalue; // retorna o valor default
      }
    catch (NumberFormatException nfe) // se houver algum erro de converso
      {
      if (debug) // se for pedida a impresso de mensagens de erro
	System.err.println("KEYBOARD:: Erro de converso de "+nt+" para um float. "+
			   "Retorna "+defaultvalue);
      return defaultvalue;
      }
    catch (NoSuchElementException nsee) // se no houver tokens
      {
      if (debug) // se for pedida a impresso de mensagens de erro
	System.err.println("KEYBOARD:: Entrada no contm um float. "+
			   "Retorna "+defaultvalue);
      return defaultvalue; // retorna o valor default
      }
    } // fim do mtodo readFloat

 /**
  * O mtodo <tt>readDouble</tt> l e retorna um valor do tipo <tt>double</tt>. 
  * Este mtodo simplesmente chama o mtodo <tt>readDouble</tt> com argumentos, 
  * descrito abaixo, considerando o valor <i>default</i> como sendo <tt>0.0</tt>.
  * @return o valor do tipo <tt>double</tt> lido (ou, em caso de erro de leitura, 
  *         <tt>0.0</tt>)
  */
  public static double readDouble() 
    {
    return readDouble(0.0);
    } // fim do mtodo readDouble

 /**
  * O mtodo <tt>readDouble</tt> l e retorna um valor do tipo <tt>double</tt>. Este
  * mtodo tenta pegar o prximo <i>token</i> a ser lido da entrada padro, 
  * analisando o valor para ver se  <tt>NaN</tt> ou infinito, e se no for, usa o 
  * valor como argumento para o mtodo <tt>parseDouble</tt> da classe <tt>Double</tt>,
  * que tenta convert-lo. Se a leitura e converso puderem ser feitas, um valor 
  * do tipo <tt>double</tt>  retornado, caso contrrio o valor <i>default</i> 
  * (passado como argumento)  retornado.
  * @param defaultvalue o valor <i>default</i> caso no seja possvel ler e 
  *                     decodificar um valor do tipo <tt>double</tt>
  * @return o valor do tipo <tt>double</tt> lido (ou, em caso de erro de leitura, 
  *         igual ao argumento passado para o mtodo)
  */
  public static double readDouble(double defaultvalue) 
    {
    try 
      {
      st = getToken(); // pega os tokens a partir da linha lida
      nt = st.nextToken(); // e a primeira string do token.
      // Verifica se o valor  aparentemente NaN ou infinito
      if (nt.toLowerCase().startsWith("nan")) return Double.NaN;
      else if (nt.toLowerCase().startsWith("inf")) return Double.POSITIVE_INFINITY;
      else if (nt.toLowerCase().startsWith("+inf")) return Double.POSITIVE_INFINITY;
      else if (nt.toLowerCase().startsWith("-inf")) return Double.NEGATIVE_INFINITY;
      // Retorna o valor processado pela classe Double
      return Double.parseDouble(nt);
      }
    catch (IOException ioe) // se houver algum erro de leitura
      {
      if (debug) // se for pedida a impresso de mensagens de erro
	System.err.println("KEYBOARD:: Erro de entrada e sada lendo um double. "+
			   "Retorna "+defaultvalue);
      return defaultvalue; // retorna o valor default
      }
    catch (NumberFormatException nfe) // se houver algum erro de converso
      {
      if (debug) // se for pedida a impresso de mensagens de erro
	System.err.println("KEYBOARD:: Erro de converso de "+nt+" para um double. "+
			   "Retorna "+defaultvalue);
      return defaultvalue;
      }
    catch (NoSuchElementException nsee) // se no houver tokens
      {
      if (debug) // se for pedida a impresso de mensagens de erro
	System.err.println("KEYBOARD:: Entrada no contm um double. "+
			   "Retorna "+defaultvalue);
      return defaultvalue; // retorna o valor default
      }
    } // fim do mtodo readDouble

 /**
  * O mtodo <tt>readChar</tt> l e retorna um valor do tipo <tt>char</tt>. Este
  * mtodo simplesmente chama o mtodo <tt>readChar</tt> com argumentos, descrito
  * abaixo, considerando o valor <i>default</i> como sendo <tt>' '</tt> (espao).
  * @return o valor do tipo <tt>char</tt> lido (ou, em caso de erro de leitura, 
  *         espao)
  */
  public static char readChar() 
    {
    return readChar(' ');
    } // fim do mtodo readChar

 /**
  * O mtodo <tt>readChar</tt> l e retorna um valor do tipo <tt>char</tt>. Este 
  * mtodo tenta pegar o prximo <i>token</i> a ser lido da entrada padro e retorna
  * o primeiro caractere deste <i>token</i>, ignorando os outros. Se a leitura puder ser 
  * feita, um valor do tipo <tt>char</tt>  retornado, caso contrrio o valor
  * <i>default</i> (passado como argumento)  retornado.
  * @param defaultvalue o valor <i>default</i> caso no seja possvel ler um valor do tipo
  *                     <tt>char</tt>
  * @return o valor do tipo <tt>char</tt> lido (ou, em caso de erro de leitura, igual ao 
  *         argumento passado para o mtodo)
  */
  public static char readChar(char defaultvalue) 
    {
    try 
      {
      st = getToken(); // pega os tokens a partir da linha lida
      nt = st.nextToken(); // e a primeira string do token.
      // retorna o primeiro caractere da string acima
      return nt.charAt(0);
      }
    catch (IOException ioe) // se houver algum erro de leitura
      {
      if (debug) // se for pedida a impresso de mensagens de erro
	System.err.println("KEYBOARD:: Erro de entrada e sada lendo um char. "+
			   "Retorna "+defaultvalue);
      return defaultvalue; // retorna o valor default
      }
    catch (NoSuchElementException nsee) // se no houver tokens
      {
      if (debug) // se for pedida a impresso de mensagens de erro
	System.err.println("KEYBOARD:: Entrada no contm um char. "+
			   "Retorna "+defaultvalue);
      return defaultvalue; // retorna o valor default
      }
    } // fim do mtodo readChar

 /**
  * O mtodo <tt>readString</tt> l e retorna uma instncia da classe <tt>String</tt>.
  * Este mtodo simplesmente chama o mtodo <tt>readString</tt> com argumentos, descrito
  * abaixo, considerando o valor <i>default</i> como sendo <tt>""</tt> (string vazia).
  * @return o valor da string lida (ou, em caso de erro de leitura, uma string
  *         vazia)
  */
  public static String readString() 
    {
    return readString("");
    } // fim do mtodo readString

 /**
  * O mtodo <tt>readString</tt> l e retorna uma instncia da classe <tt>String</tt>. 
  * Este mtodo retorna uma linha inteira lida da entrada padro, sem process-la como
  * <i>tokens</i>. Se a leitura puder ser feita, a string lida  retornada, caso 
  * contrrio o valor <i>default</i> (passado como argumento)  retornado. O valor
  * <i>default</i> tambm  retornado no caso de uma string vazia. 
  * @param defaultvalue o valor <i>default</i> caso no seja possvel ler uma instncia
  *                     da classe <tt>String</tt>
  * @return o valor da string lida (ou, em caso de erro de leitura, igual ao 
  *         argumento passado para o mtodo)
  */
  public static String readString(String defaultvalue) 
    {
    try 
      {
      nt = br.readLine(); // uma string lida diretamente 
      if (nt.trim().length() == 0) // nada foi entrado, ento
	return defaultvalue; // retorna o valor default
      else return nt; // retorna o que foi lido
      }
    catch (IOException ioe) // se houver algum erro de leitura
      {
      if (debug) // se for pedida a impresso de mensagens de erro
	System.err.println("KEYBOARD:: Erro de entrada e sada lendo uma string. "+
			   "Retorna "+defaultvalue);
      return defaultvalue; // retorna o valor default
      }
    } // fim do mtodo readString

 /**
  * O mtodo <tt>debugOn</tt> modifica o campo que indica que mensagens de erro 
  * devero ser mostradas, fazendo com que sejam mostradas at que o mtodo 
  * <tt>debugOff</tt> seja chamado.
  */
  public static void debugOn() 
    {
    debug = true;
    System.err.println("KEYBOARD:: Mostrando mensagens de erro e avisos...");
    } // fim do mtodo debugOn

 /**
  * O mtodo <tt>debugOff</tt> modifica o campo que indica que mensagens de erro 
  * devero ser mostradas, fazendo com que no sejam mostradas at que o mtodo 
  * <tt>debugOn</tt> seja chamado.
  */
  public static void debugOff() 
    {
    debug = false;
    } // fim do mtodo debugOff

  } // fim da classe Keyboard

// Segue a mensagem original da classe EasyIn de Peter van der Linden

// Simple input from the keyboard for all primitive types. ver 1.0
// Copyright (c) Peter van der Linden,  May 5 1997.
//     corrected error message 11/21/97
//
// The creator of this software hereby gives you permission to:
//  1. copy the work without changing it
//  2. modify the work providing you send me a copy which I can
//     use in any way I want, including incorporating into this work.
//  3. distribute copies of the work to the public by sale, lease, 
//     rental, or lending
//  4. perform the work
//  5. display the work
//  6. fold the work into a funny hat and wear it on your head.
//
// This is not thread safe, not high performance, and doesn't tell EOF.
// It's intended for low-volume easy keyboard input.
// An example of use is:
//     EasyIn easy = new EasyIn();
//     int i = easy.readInt();   // reads an int from System.in
//     float f = easy.readFloat();   // reads a float from System.in
