//
//   This file contains the Java code from Program 6.2 of
//   "Data Structures and Algorithms
//    with Object-Oriented Design Patterns in Java"
//   by Bruno R. Preiss.
//
//   Copyright (c) 1998 by Bruno R. Preiss, P.Eng.  All rights reserved.
//
//   http://www.pads.uwaterloo.ca/Bruno.Preiss/books/opus5/programs/pgm06_02.txt
//

package pilhaA;

import java.util.NoSuchElementException;
import fundamentos.AbstractContainer;
import fundamentos.Comparable;
import fundamentos.ContainerEmptyException;
import fundamentos.Enumeration;
import fundamentos.Visitor;
import fundamentos.ContainerFullException;

public class StackAsArray
    extends AbstractContainer
    implements Stack
{
    protected Object[] array;

	public StackAsArray (int size)
	{ array = new Object [size]; }

    public void purge ()
    {
	while (count > 0)
	    array [--count] = null;
    }
    
    public void push (Object object) throws ContainerFullException 
    {
        if (count == array.length)
            throw new ContainerFullException ("Pilha cheia!");            
        array [count++] = object;
    }

    public Object pop () throws ContainerEmptyException 
    {
        if (count == 0)
            throw new ContainerEmptyException ("Pilha vazia!");            
        Object result = array [--count];
        array [count] = null;
        return result;
    }

    public Object getTop () throws ContainerEmptyException
    {
        if (count == 0)
            throw new ContainerEmptyException ("Pilha vazia!");            
        return array [count - 1];
    }
    
    public void accept (Visitor visitor)
    {
        for (int i = 0; i < count; ++i)
        {
            visitor.visit (array [i]);
            if (visitor.isDone ())
                return;
        }
    }
    
    public Enumeration getEnumeration ()
    {
        return new Enumeration ()
        {
            protected int position = 0;
            
            public boolean hasMoreElements ()
            	{ return position < getCount (); }
            
            public Object nextElement ()
            {
                if (position >= getCount ())
                    throw new NoSuchElementException ();
                return array [position++];
            }
        };
    }

    /* (No-Javadoc)
     * @see fundamentos.AbstractObject#compareTo(fundamentos.Comparable)
     */
    protected int compareTo(Comparable arg)
    {
        // TODO Mtodo gerado automaticamente.
        return 0;
    }
    
    public String toString()
    {
        String stackAsArray;
        
        if(count == 0)
            stackAsArray = "Pilha vazia.";
        else
        {    
            stackAsArray = "\n\n\t\tPilha Final\n\n";
            for(int i = array.length-1; i >= 0; i--)
            {
                if(array[i] != null)
                    stackAsArray += "" + array[i];
            }
        }
        
        return stackAsArray;
    }
}
